/*
 * debug.h: log (or not) messages using syslog
 * Copyright (C) 2001 Ludovic Rousseau <ludovic.rousseau@free.fr>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free 
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 59 
 * Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * 
 */

/*
 * $Id
 */

#ifdef DEBUG
void DEBUG_MSG(char *fmt, ...);
void DEBUG_XXD(char *msg, unsigned char *buffer, int size);
#else
#define DEBUG_MSG(fmt, ...)
#define DEBUG_XXD(msg, buffer, size)
#endif
