
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _defines_h_
#define _defines_h_

/*
 * $Id: defines.h,v 1.1.1.1 2001/09/07 20:19:18 rousseau Exp $
 *
 * NAME:
 *  defines.h -- Copyright (C) 1998 David Corcoran
 *                corcordt@cs.purdue.edu
 *
 * DESCRIPTION:
 *       Some global definitions and typedefs
 *
 * AUTHOR:
 *  David Corcoran, 3/17/98
 *
 *  Modified for Macintosh by Mark Hartman
 */

/*
 * Boolean constants
 */

#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif

/*
 * Type definitions
 */

#ifndef _BYTE
typedef unsigned char BYTE;
#endif

#ifndef HANDLE

#ifdef CPU_MAC_PPC
typedef Handle HANDLE;
#endif

#ifdef CPU_ICAP_PC
typedef int HANDLE;
#endif

#ifdef CPU_SUN_SPARC
typedef int HANDLE;
#endif

#endif

#ifndef bool
typedef short bool;
#endif

/*
 * Simple Max Defines
 */

#define MAX_BUFFER_SIZE    264
#define STATUS_SIZE        2

/*
 * Compiler dependencies
 */

#ifdef __GNUC__
#define UNUSED __attribute__((unused))
#else
#define UNUSED
#endif

#endif
